By: Greg Smith
Re: ARJ Viewer 1 of 7
------------------------------------------------------------
{ ARJ Viewing Unit.  Based on the C code of AV.C included with the ARJ
  archiver program.  Released into the Public Domain.  -- Greg Smith
  Please Include a revision history below if you modify and distribute it:
    09-08-91  Inital release into public - Greg Smith

 }

Unit ARJView; { Code for viewing ARJ files }
{$X+,S+,V-,R-,D-,L-}
{$IFDEF OVERLAY}
{$F+,O+}
{$ELSE}
{$F-,O-}
{$ENDIF}

interface

uses Dos;

{$IFNDEF AV}
type
  CompRec = record
    Name   : string[12];
    Path   : string;
    Attr   : byte;
    Time   : string[8];
    Date   : string[8];
    CSize  : longint;
    UCSize : longint;
    CRC    : string[8];
    Flags  : string[6]; { TPMGVX }
  end;
{$ENDIF}

const
  ShowPaths : boolean = False;
{$IFNDEF AV}  ShowTotals : boolean = True; {$ENDIF}
  ShowCRCs : boolean = True;
  ShowFlags : boolean = True; { TPMGVX or COMP# }

(*   Pass the Text File for which output should go through
     and the ARJfile of byte variable.  This function will
     display the contents and update several flags according
     to the contents of the file.  a non zero result is
     returned in the event of an error                       *)

Function View_ARJ(var out:text;var ARJfile:file): integer;

{$IFNDEF AV}
Function InitARJ(var ARJfile:file): integer; { init file for find below }

{ Get the next file in the archive }
Function FindFileARJ(var CR:CompRec;var ARJfile:file): integer;
{$ENDIF}

implementation

type
  PreBasicHead = record
    HeadID : word; { $EA60 }
    BSize : word; { Basic Header Size
          (from first_hdr_size thru comment)
          = First_hdr_size + zStrLen(FileName) + 1 + StrLen(Comment) + 1
          = 0 if end of archive             }
  end; { PreBasicHead }
  BasicHdr = record { continued from Pre-Header }
    First_hdr_size : byte; { size up to Extra_Data }
    ARJ_Version : byte; { Archiver version number }
    ARJ_Extract : byte; { Minimum archive version to extract }
    Host_OS : byte; { 0=MSDOS 1=PRIMOS 2=UNIX 3=AMIGA 4=MACDOS }
    ARJ_Flags : byte; { bits:
                             $01 = Garbled with Password
                             $02 = Reserved for ???
                             $04 = Volume_Flag, file continued in next volume
                             $08 = ExtFile_Flag, file start position field
                             $10 = Path_Sym, indicates path translated }
    Method : byte; { 0 stored, 1=most ... 4=fastest }
    File_Type : byte; { 0=binary 1=text file 2=comment }
    Reserved1 : byte; { Reserved for ??? }
    Date_Time : longint; { Date & Time stamp.  see end }
    Comp_Size : longint; { Compressed Data Size }
    Orig_Size : longint; { Original file size }
    File_CRC : longint; { Original File CRC }
    Entry_Pos : word; { Entry name position in file name }
    File_Mode : word; { File Access mode (dos flags or HSRA) }
    Host_Data : word; { Currently Unused }
  end; { Common Basic Header Record }
(*          ? bytes of extra data containing:
(*            longint : Extended File Position if ExtFile_Flag set
(*            FileName : ASCIIZ;
(*            Comment : ASCIIZ;
(*            BHead_CRC : longint; { Basic Header CRC }
(*            Ext_Head1Size : word; { 1st extended header size (0=none) }
(*            Ext_Head1Size bytes of 1st extended header
(*            Ext_Head1CRC : longint; { CRC of 1st extended header }
(*            ...
(*            ? bytes of Compressed File (Comp_Size bytes?)
(*         end;  *)

const
  HeaderID = $EA60; { 60000 }
  Garbled = $01;
  Volume = $04;
  ExtFile = $08;
  Path_Sym = $10;
  MaxSFX = 25000; { Maximum bytes of SFX code }
  FName_Max = 512; { Maximum file name length }
  First_Hdr_Size = 30;
  Comment_Max = 2048;
  HeaderSize_Max = First_Hdr_Size + 10 + FName_Max + Comment_Max;
  OS = 0; { MS-Dos }
{$IFNDEF AV}
  Host_OS_Names : Array[0..5] of string[6] = ('MS-DOS','PRIMOS','UNIX','AMIGA',
'MAC-OS',#0);
{$ENDIF}
  Binary_type = 0;
  Text_type = 1;
  Comment_Type = 2;
  Dir_type = 3;
  ASCII_Mask = $7F;
  CRC_Mask = $FFFFFFFF;
(*  CRC_Poly = $EDB88320; { Seed for CRC-32 calculations } *)
  CRCTable : Array[0..255] of longint = (                 { table for CRC
Polynomials }
$00000000, $77073096, $ee0e612c, $990951ba, $076dc419, $706af48f, $e963a535,
$9e6495a3,
$0edb8832, $79dcb8a4, $e0d5e91e, $97d2d988, $09b64c2b, $7eb17cbd, $e7b82d07,
$90bf1d91,
$1db71064, $6ab020f2, $f3b97148, $84be41de, $1adad47d, $6ddde4eb, $f4d4b551,
$83d385c7,
$136c9856, $646ba8c0, $fd62f97a, $8a65c9ec, $14015c4f, $63066cd9, $fa0f3d63,
$8d080df5,
$3b6e20c8, $4c69105e, $d56041e4, $a2677172, $3c03e4d1, $4b04d447, $d20d85fd,
$a50ab56b,
$35b5a8fa, $42b2986c, $dbbbc9d6, $acbcf940, $32d86ce3, $45df5c75, $dcd60dcf,
$abd13d59,
$26d930ac, $51de003a, $c8d75180, $bfd06116, $21b4f4b5, $56b3c423, $cfba9599,
$b8bda50f,
$2802b89e, $5f058808, $c60cd9b2, $b10be924, $2f6f7c87, $58684c11, $c1611dab,
$b6662d3d,
$76dc4190, $01db7106, $98d220bc, $efd5102a, $71b18589, $06b6b51f, $9fbfe4a5,
$e8b8d433,
$7807c9a2, $0f00f934, $9609a88e, $e10e9818, $7f6a0dbb, $086d3d2d, $91646c97,
$e6635c01,
$6b6b51f4, $1c6c6162, $856530d8, $f262004e, $6c0695ed, $1b01a57b, $8208f4c1,
$f50fc457,
$65b0d9c6, $12b7e950, $8bbeb8ea, $fcb9887c, $62dd1ddf, $15da2d49, $8cd37cf3,
$fbd44c65,
$4db26158, $3ab551ce, $a3bc0074, $d4bb30e2, $4adfa541, $3dd895d7, $a4d1c46d,
$d3d6f4fb,
$4369e96a, $346ed9fc, $ad678846, $da60b8d0, $44042d73, $33031de5, $aa0a4c5f,
$dd0d7cc9,
$5005713c, $270241aa, $be0b1010, $c90c2086, $5768b525, $206f85b3, $b966d409,
$ce61e49f,
$5edef90e, $29d9c998, $b0d09822, $c7d7a8b4, $59b33d17, $2eb40d81, $b7bd5c3b,
$c0ba6cad,
$edb88320, $9abfb3b6, $03b6e20c, $74b1d29a, $ead54739, $9dd277af, $04db2615,
$73dc1683,
$e3630b12, $94643b84, $0d6d6a3e, $7a6a5aa8, $e40ecf0b, $9309ff9d, $0a00ae27,
$7d079eb1,
$f00f9344, $8708a3d2, $1e01f268, $6906c2fe, $f762575d, $806567cb, $196c3671,
$6e6b06e7,
$fed41b76, $89d32be0, $10da7a5a, $67dd4acc, $f9b9df6f, $8ebeeff9, $17b7be43,
$60b08ed5,
$d6d6a3e8, $a1d1937e, $38d8c2c4, $4fdff252, $d1bb67f1, $a6bc5767, $3fb506dd,
$48b2364b,
$d80d2bda, $af0a1b4c, $36034af6, $41047a60, $df60efc3, $a867df55, $316e8eef,
$4669be79,
$cb61b38c, $bc66831a, $256fd2a0, $5268e236, $cc0c7795, $bb0b4703, $220216b9,
$5505262f,
$c5ba3bbe, $b2bd0b28, $2bb45a92, $5cb36a04, $c2d7ffa7, $b5d0cf31, $2cd99e8b,
$5bdeae1d,
$9b64c2b0, $ec63f226, $756aa39c, $026d930a, $9c0906a9, $eb0e363f, $72076785,
$05005713,
$95bf4a82, $e2b87a14, $7bb12bae, $0cb61b38, $92d28e9b, $e5d5be0d, $7cdcefb7,
$0bdbdf21,
$86d3d2d4, $f1d4e242, $68ddb3f8, $1fda836e, $81be16cd, $f6b9265b, $6fb077e1,
$18b74777,
$88085ae6, $ff0f6a70, $66063bca, $11010b5c, $8f659eff, $f862ae69, $616bffd3,
$166ccf45,
$a00ae278, $d70dd2ee, $4e048354, $3903b3c2, $a7672661, $d06016f7, $4969474d,
$3e6e77db,
$aed16a4a, $d9d65adc, $40df0b66, $37d83bf0, $a9bcae53, $debb9ec5, $47b2cf7f,
$30b5ffe9,
$bdbdf21c, $cabac28a, $53b39330, $24b4a3a6, $bad03605, $cdd70693, $54de5729,
$23d967bf,
$b3667a2e, $c4614ab8, $5d681b02, $2a6f2b94, $b40bbe37, $c30c8ea1, $5a05df1b,
$2d02ef8d
);
  GlobalError : integer = 0; { Set by a function when an error occurs }
  Header_CRC : longint = CRC_Mask;
  crc : longint = CRC_Mask;
  FileName : integer = 0; { Pointer into Header buffer, ASCIIZ }
  Comment : integer = 0; { Pointer into Header buffer, ASCIIZ }
  ExtHeaderSize : word = 0;
  torigsize : longint = 0; { Total Original Sizes }
  tcompsize : longint = 0; { Total Compressed Sizes }
  hs : string[80] = 'Filename       Original Compressed Ratio   Date  &  Time
Attr';
  ls : string[80] = '      ';

var
  PH : PreBasicHead; { ID and Size of Basic header }
  Header : Array[0..HeaderSize_Max] of char;
  H : BasicHdr absolute Header; { ARJ header data in array }

(*   Now using a pre-calculated one
Procedure Make_CRCTable;
var
  i,j : integer;
  r : longint;
begin
  for i := 0 to 255 do       { 255 = CHAR_MAX }
   begin
     r := i;
     for j := 0 to 7 do      { 7 = CHAR_BIT }
      if (r AND 1) <> 0 then
        r := (r shr 1) XOR CRC_Poly
       else
        r := r shr 1;
     CRCTable[i] := r;
   end; { for i }
end; { Make_CRCTable }
*)

Procedure Update_CRC(c:byte;var CRC:longint);
begin
  crc := CRCTable[Byte(crc XOR LongInt(c))] XOR ((crc shr 8));
end;

Procedure StriParity(n:integer); { used on file name & comment. ASCIIZ only }
var               { n is index into header buffer }
  c : char;
begin
  c := Header[n];
  while c <> #0 do
   begin
     Header[n] := chr(ord(Header[n]) AND ASCII_Mask);
     inc(n);
     c := Header[n];
   end;
end; { StriParity }

Procedure DecodePath(n:integer); { used on filename. index in header. ASCIIZ }
var
  c : char;
begin
  c := Header[n];
  while c <> #0 do
   begin
     if Header[n] = '/' then Header[n] := '\'; { change path character }
     inc(n);
     c := Header[n];
   end;
end; { DecodePath }

Function ZLength(n:integer): word; { length of ASCIIZ at Header[n] }
var
  z : word;
begin
  z := 0;
  while Header[n] <> #0 do
   begin
     inc(n);
     inc(z);
   end;
  Zlength := z;
end;

Procedure CRC_buf(var str;len:word);
var
  z : word;
begin
  if len = 0 then exit;
  for z := Ofs(str) to Ofs(str)+pred(len) do
   begin
     Update_CRC(Mem[Seg(str):z],crc);
   end;
end;

Function FRead_CRC(var ch;n:word;var f:file): integer;
begin
  {$I-}
  BlockRead(f,Mem[Seg(ch):Ofs(ch)],n,n);
  {$I+}
  GlobalError := IOResult;
  CRC_buf(Mem[Seg(ch):Ofs(ch)],n);
  FRead_CRC := n;
end;

Function FGet_CRC(var f:file): longint;
var
  z : longint;
begin
  {$I-}
  BlockRead(f,z,4);
  {$I+}
  GlobalError := IOResult;
  FGet_CRC := z;
end;

Function Ratio(a,b:longint): byte;
begin
  Ratio := round((a / b) * 100);
end; { Ratio }

Procedure Date_To_Str(var ds:string;t:longint);
var
  dt : DateTime;
  s,r : string[20];
begin
  UnpackTime(t,dt);
  with dt do
   begin
     Str(Year:2,s);
     Str(Month:2,r);
     s := s+'-'+r;
     Str(Day:2,r);
     s := s+'-'+r;
     Str(Hour:2,r);
     s := s+'.'+r; { change to space later }
     Str(Min:2,r);
     s := s+':'+r;
     Str(Sec:2,r);
     s := s+':'+r;
   end;
  While Pos(' ',s) <> 0 do
   s[Pos(' ',s)] := '0'; { blanks to zeros }
  Delete(s,1,2); { remove century qualifier }
  s[Pos('.',s)] := ' '; { change . to space }
  ds := s;
end;

Procedure Get_Mode_Str(var m:string;mode:word);
begin
  m := '---W';
  if (mode AND Archive) > 0 then
   m[1] := 'A';
  if (mode AND SysFile) > 0 then
   m[2] := 'S';
  if (mode AND Hidden) > 0 then
   m[3] := 'H';
  if (mode AND ReadOnly) > 0 then
   m[4] := 'R';
end;

Procedure ZWrite(var o:text;n,m:integer); { n is index into header }
var               { ASCIIZ Write with m minimum characters }
  s : string;
  n1 : integer;
begin
  n1 := n;
  s[0] := #254;
  Repeat
    Move(header[n],s[1],254);
    s[0] := chr( Pred(Pos(#0,s)) );
    if s[0] = #0 then s[0] := #255;
    Write(o,s);
    inc(n,ord(s[0]));
  Until s[0] <> #255;
  if n1 <> n then
   if ord(s[0]) < m then
    begin
      FillChar(s[1],m - ord(s[0]),' '); { pad with spaces }
      s[0] := chr(m - ord(s[0]));
      Write(s);
    end; { if(s) }
end;

Function Long2Hex(l:longint;n:byte): string; { returns hex string for longint }
var
  s : string[8];
  b,c,d : byte;
const
  HD : Array[0..15] of char = ('0','1','2','3','4','5','6','7','8','9','a',
                               'b','c','d','e','f');
begin
  s := '00000000';
  s[0] := chr(n);
  b := 1;
  for b := n downto 1 do
   begin
     s[b] := HD[l AND $000F];
     l := l shr 4;
   end;
  Long2Hex := s;
end;

Function Find_Header(var f:file): longint; { returns position of header }
const
  bs = 1023; { size of buffer }
type
  BufArr = Array[0..bs] of byte; { input & scan buffer }
  ArrPtr = ^BufArr;
label
  break1, break2, loopback;
var
  c,i,z : integer;
  o,arcpos : longint;
  bp : ArrPtr;
begin
  New(bp);
  if bp = nil then
   begin
     Find_Header := 0;
     exit; { not enough memory for efficient buffer }
   end;
  i := 0;
  BlockRead(f,bp^,succ(bs),z);
  for arcpos := 0 to pred(MaxSFX) do
   begin
 loopback:
     c := arcpos-(i*bs);
     while c < z do
      if bp^[c] <> lo(HeaderID) then { low order byte first! (normal) }
       begin
         inc(arcpos);
         inc(c); { c := arcpos-(i*bs) }
       end
      else { if else }
       begin
         if bp^[succ(c)] = hi(HeaderID) then goto break1;  { ID Found }
         inc(arcpos);
         inc(c); { c := arcpos-(i*bs) }
       end; { else & while }
     if EOF(f) or (FilePos(f) >= MaxSFX) then
      begin
        Find_Header := 0;
        exit; { end of file }
      end;

     Seek(f,pred(FilePos(f))); { not to break a hdr between buffers }
     dec(arcpos); { arcpos-- because 1 byte fall back }
     BlockRead(f,bp^,succ(bs),z); { reload buffer }
     inc(i);
     goto loopback; { buffer refilled, go back to loop }
 break1:
     o := FilePos(f); { save incase of a dud }
     Seek(f,arcpos+2); { skip Header ID }
     BlockRead(f,PH.bsize,2);
     if (PH.bsize > 0) AND (PH.bsize <= HeaderSize_Max) then
      begin
        crc := CRC_Mask;
        FRead_CRC(H,PH.bsize,f);
        if (crc XOR CRC_Mask) = FGet_CRC(f) then
         begin
           Seek(f,arcpos);
           goto break2;
         end;
      end; { 2nd if }
     Seek(f,o); { restore old position }
   end; { for }
 break2:
  Find_Header := arcpos; { position of header }
  Dispose(bp);
end; { Find_Header }

Function Read_Header(var f:file): boolean;
var
  extheadsize : byte;
begin
  {$I-}
  BlockRead(f,PH,4);
  {$I+}
  GlobalError := IOResult;
  If PH.HeadID <> HeaderID then
   begin
     GlobalError := -3; { Bad header }
     Read_Header := False;
     exit;
   end;
  if PH.bsize = 0 then
   begin
     Read_Header := False; { End of archive }
     exit;
   end;
  if PH.bsize > HeaderSize_Max then
   begin
     Read_Header := False;
     GlobalError := -4; { Bad Header (size) }
     exit;
   end;
  crc := CRC_Mask;
  FRead_CRC(Header,PH.bsize,f); { CRC the header }
  {$I-}
  BlockRead(f,Header_CRC,4); { Read the CRC of the header }
  {$I+}
  GlobalError := IOResult;
  if (crc XOR CRC_Mask) <> Header_CRC then
   begin
     Read_Header := False; { CRC Error }
     GlobalError := -5; { Header CRC Error }
     exit;
   end;

  FileName := SizeOf(H); { index to filename in Header }
  if H.Host_OS <> OS then
   StriParity(FileName);
  if (H.ARJ_flags AND Path_Sym) <> 0 then
   DecodePath(FileName);

  Comment := SizeOf(H)+1+ZLength(FileName); { index to Comment in header }
  if H.Host_OS <> OS then
   StriParity(Comment);

  {$I-}
  BlockRead(f,Extheadersize,2);
  {$I+}
  GlobalError := IOResult;
  while extheadersize <> 0 do
   begin
     {$I-}
     seek(f,extheadersize+2+FilePos(f)); { skip extended headers }
     BlockRead(f,Extheadersize,2);
     {$I+}
     GlobalError := IOResult;
   end;

  Read_Header := True; { success }
end;

Procedure Skip(var f:file); { skip compressed data }
begin
  seek(f,H.comp_size+FilePos(f));
end;

Procedure List_Start(var o:text);
begin
  if ShowCRCs then
   begin
     insert('CRC-32   ',hs,60);
     insert(' ',ls,60);
   end;
  if ShowFlags then hs := hs + ' TPMGVX'
   else hs := hs + ' Method';
  Writeln(o,hs);
  Writeln(o,ls);
end;

Function Flag_Str: string;
const
  mode : Array[0..3] of char = ('B','T','?','D');
  grbf : Array[0..1] of char = (' ','G');
  pthf : Array[0..1] of char = (' ','+');
  volf : Array[0..1] of char = (' ','V');
  exta : Array[0..1] of char = (' ','X');
var
  garb,path,volm,extf,ftype : byte;
  FlagStr : string[6];
begin
  with H do
   begin
     garb := ARJ_Flags AND Garbled;
     if entry_pos > 0 then path := 1 else path := 0;
     volm := (ARJ_Flags AND Volume) shr 2;
     extf := (ARJ_Flags AND ExtFile) shr 4;
     ftype := File_Type;
     if (ftype <> Binary_Type) AND (ftype <> Text_Type) AND (ftype <> Dir_Type)
then
      ftype := 3;

     Str(Method:1,FlagStr);
     if ShowFlags then
      begin
        Insert(mode[ftype]+pthf[path],FlagStr,1);
        FlagStr := FlagStr+grbf[garb]+volf[volm]+exta[extf];
      end
     else
      begin
        Insert('COMP',FlagStr,1);
        if Method = 0 then FlagStr := 'STORED';
      end;
   end; { with H }
  Flag_Str := FlagStr;
end;

Procedure List_Arc(var o:text;count:integer);
var
  r : byte;
  date,fm : string[20];
  fcrc_str: string[9]; { Hex string containing file_crc }
begin
  if count = 0 then List_Start(o);
  with H do
   begin
     r := ratio(comp_size,orig_size);
     inc(torigsize,orig_size);
     inc(tcompsize,comp_size);
     if ShowCRCs then fcrc_str := Long2Hex(File_CRC,8)+' '
      else fcrc_str := '';
     Date_To_Str(date,Date_Time);
     fm := '';
     if Host_OS = OS then Get_Mode_Str(fm,File_Mode);
     if ZLength(FileName) > 12 then
      begin
        if ShowPaths then
         begin
           Zwrite(o,FileName,0);
           Write(#13,#10,'            ');
         end
        else
         Zwrite(o,FileName+entry_pos,12);
      end
     else
      ZWrite(o,FileName,12);
     Writeln(' ',orig_size:10,' ',comp_size:10,' ',r:4,'% ',date,' ',
           fcrc_str,fm,' ',Flag_Str);
   end; { with }
end; { List_Arc }

Function View_ARJ(var out:text;var ARJfile:file) : integer;
var
  count : integer;
begin
  {$I-}
  Find_Header(ARJfile);
  {$I+}
  GlobalError := IOResult;
  if GlobalError <> 0 then
   begin
     View_ARJ := GlobalError;
     exit;
   end;
  if Not Read_Header(ARJfile) then
   begin
     View_ARJ := GlobalError;
     exit;
   end;

  count := 0;
  while read_header(ARJfile) do
   begin
     if GlobalError <> 0 then
      begin
        View_ARJ := GlobalError;
        exit;
      end;
     list_arc(out,count);
     inc(count);
     skip(ARJfile);
   end;

  Writeln(out,ls);
{$IFNDEF AV}  if ShowTotals then {$ENDIF}
   Writeln(out,'',count:4,' files   ',torigsize:10,' ',tcompsize:10,' ',
Ratio(tcompsize,torigsize):4,'%');
  View_ARJ := 0; { error code 0 }
end;

{$IFNDEF AV}
Function InitARJ(var ARJfile:file): integer;
begin
  {$I-}
  Find_Header(ARJfile);
  {$I+}
  InitARJ := IOResult;
end;

Function FindFileARJ(var CR:CompRec;var ARJfile:file): integer;
var
  i : integer;
  ds : string[17];
begin
  FillChar(CR,SizeOf(CR),#0); { initalize it to nothing }
  if not Read_Header(ARJfile) then
   begin
     FindFileARJ := GlobalError;
     exit;
   end;
  Skip(ARJFile); { goto next header }
  with H do
   begin
     with CR do
      begin
        Name[0] := #12;
        Move(Header[FileName+entry_pos],Name[1],12);
        i := Pos(#0,Name);
        if i <> 0 then Name[0] := chr(i);
        Path[0] := #255;
        Move(Header[FileName],Path[1],255);
        if (Zlength(FileName)-Zlength(FileName+entry_pos)) > 255 then
         Path[255] := '+'
        else Path[0] := chr(Zlength(Filename)-Zlength(filename+entry_pos));
        Attr := lo(FileMode);
        Date_To_Str(ds,Date_Time);
        Move(ds[1],Date[1],8);
        Date[0] := #8;
        Move(ds[9],Time[1],8);
        Time[0] := #8;
        CRC := Long2Hex(File_CRC,8);
        CSize := Comp_Size;
        UCSize := Orig_Size;
        Flags := Flag_Str;
      end; { with CR }
   end; { with H }
  FindFileARJ := 0; { Success }
end; { FindFileARJ }
{$ENDIF}


END.  { End of unit }

(*
This is a demo of the previous ARJ-View unit.  The unit and this were written
for Turbo Pascal 6.0, but adapting them for 5.5 or 5.0 shouldn't be to hard for
you to do.

-+- cut ---------- *)

Program ARJViewer; { View ARJ Archive Contents }

{$X+,D-,L-,S+,R-,V-}
{$M 2500,1344,1344}

Uses Dos, ArjView;

var
  fname : PathStr;
  f : file;
  r : integer;
  s : String[127];
  ep : pointer;

Procedure eclose; far;
begin
  Close(f);
  ExitProc := ep;
end;

begin
  if paramcount > 1 then
   for r := 2 to paramcount do
    begin
      s := paramstr(r);
      if (ord(s[0]) > 1) AND (s[1] = '/') then
       case s[2] of
         'P','p' : ShowPaths := True;
         'R','r' : ShowCRCs := False;
         'F','f' : ShowFlags := False;
       end;
    end;
  if paramstr(1) = '' then
   begin
     Writeln('ARJ Viewer Pascal 1.31 -- Copyright (c) 1991 Greg Smith');
     Writeln('Converted from AV.C Ver 1.2 (c) 1990 Robert K Jung');
     Writeln;
     Writeln('  Usage: AV <filename> [parameter(s)]');
     Writeln('   /p = show paths      /r = no CRCs      /f = no flags');
     halt;
   end;
  fname := paramstr(1);
  if Pos('.',fname) = 0 then fname := fname +'.ARJ';
  Assign(f,fname);
  Reset(f,1);
  ep := ExitProc;
  ExitProc := @eClose;
  Writeln(#13,#10,'Viewing: ',fname);
  r := View_ARJ(output,f);
  case r of
     0 : { good } ;
    -3 : Writeln('Bad Header');
    -4 : Writeln('Bad Header Size');
    -5 : Writeln('Bad Header CRC');
  else
   Writeln('Unknown error #',r);
  end;
  Close(f);
  ExitProc := ep;
end.
